#!/bin/bash
#
# build_lapack.sh 
# (in source build required)

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e4:prof|e5:prof|e6:prof|e7:prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir()
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}:${PRODUCTS}
}

set_qualifiers()
{

    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`
    qualver=`echo ${basequal} | sed -e 's/:/_/g'`
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
      (( darwin = 1 ))
      flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}/src
}

function die()
{
  echo "$1"; exit 2
}

function setup_and_verify()
{
    # this should not complain
    echo "Finished building ${package} ${pkgver}"
    setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
    echo "${package} is installed at ${LAPACK_FQ_DIR}"
}

function make_osx_prodct_stub()
{
    cd ${pkgdir}/lib/
#create softlinks here
    blas_path=/System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/A/libBLAS.dylib
    lapack_path=/System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/A/libLAPACK.dylib

    ln -s ${blas_path} libblas.dylib
    ln -s ${lapack_path} liblapack.dylib

    ${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

    setup_and_verify

    # this must be last
    if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
    then
       ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=lapack
origpkgver=v3_5_0
pkgver=${origpkgver}
ssibuildshims_version=v0_08_06
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtar=lapack-${pkgdotver}.tgz

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}/lib
if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   exit 1
fi

# make a product stub for OSX
if [ "${OS1}" = "Darwin" ]
then
    make_osx_prodct_stub
    exit 0
fi

# Nothing below is run on OSX...

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# lapack's makefiles can not do an out-of-source build.
cd ${sourcedir} || die "Unable to cd to ${sourcedir}"
echo "In code directory: ${PWD}"
tar xf ${tardir}/${pkgtar} || die "Unable to unwind ${tardir}/${pkgtar} into ${PWD}"

# Apply the patch.
#osname=`uname -s`
#patch -p1 < ${patchdir}/${osname}.patch || die "Failed to apply patch"

# We only make an optimized build, because lapack does not support
# anything else (and nobody would really want it anyway).
cd ${sourcedir}/lapack-${pkgdotver} || die "Unable to cd to ${sourcedir}/lapack-${pkgdotver}"

# Rename the patched example, make.inc.example, to make.inc
mv make.inc.example make.inc

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup 
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

set -x

cd ${sourcedir}/lapack-${pkgdotver} || die "Unable to cd to ${sourcedir}/lapack-${pkgdotver}"

setup cmake v3_0_1 || die "Unable to setup cmake"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

env CC=gcc CXX=g++ FC=gfortran cmake -DCMAKE_INSTALL_PREFIX=${LAPACK_FQ_DIR} \
                                     -DCMAKE_BUILD_TYPE=RELEASE \
                                     -DBUILD_SHARED_LIBS=ON \
                                     -DBUILD_STATIC_LIBS=OFF \
                                     ../lapack-3.5.0 || die "Failed to run CMake"
make -j $ncore || die "Failed in 'make'"
ctest -j $ncore || die "Failed in ctest"
make install || die "Failed to install"
#make -j $ncore lib || die "Failed in 'make lib'"
#make blas_testing || die "Failed in 'make blas_testing'"
#make lapack_testing || die "Failed in 'make lapack_testing'"


# Move libraries to the right place.
make install
#cp *.so ${pkgdir}/lib/

# Create symbolic links
pushd ${pkgdir}/lib
ln -s liblapack.so liblapack.so.3
if [ ! -L liblapack.so.3 ]
then
    echo "ERROR: failed to create symbolic link liblapack.so.3 in ${pkgdir}/lib"
    exit 1
fi
popd

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

setup_and_verify

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
